@@ -14,8 +14,8 @@ extern crate thiserror;
 use rand::{rngs::SmallRng, RngCore, SeedableRng};
 use solana_rbpf::{
     assembler::assemble,
-    ebpf::{self, hash_symbol_name},
-    elf::ElfError,
+    ebpf,
+    elf::{register_bpf_function, ElfError},
     error::EbpfError,
     memory_region::AccessType,
     syscalls,
@@ -23,7 +23,7 @@ use solana_rbpf::{
     verifier::check,
     vm::{Config, DefaultInstructionMeter, EbpfVm, Executable, SyscallObject, SyscallRegistry},
 };
-use std::{fs::File, io::Read};
+use std::{collections::BTreeMap, fs::File, io::Read};
 use test_utils::{
     BpfSyscallString, BpfSyscallU64, BpfTracePrintf, Result, SyscallWithContext,
     TestInstructionMeter, PROG_TCP_PORT_80, TCP_SACK_ASM, TCP_SACK_MATCH, TCP_SACK_NOMATCH,
@@ -31,24 +31,20 @@ use test_utils::{
 
 macro_rules! test_interpreter_and_jit {
     (register, $executable:expr, $syscall_registry:expr, $location:expr => $syscall_function:expr; $syscall_context_object:expr) => {
-        $syscall_registry.register_syscall_by_hash::<UserError, _>($location, $syscall_function).unwrap();
-    };
-    (register, $executable:expr, $syscall_registry:expr, $location:expr => $function_pc:expr) => {
-        $executable.register_bpf_function($location, $function_pc);
+        $syscall_registry.register_syscall_by_name::<UserError, _>($location, $syscall_function).unwrap();
     };
     (bind, $vm:expr, $location:expr => $syscall_function:expr; $syscall_context_object:expr) => {
         $vm.bind_syscall_context_object(Box::new($syscall_context_object), None).unwrap();
     };
-    (bind, $vm:expr, $location:expr => $function_pc:expr) => {};
-    ($executable:expr, $mem:tt, ($($location:expr => $a:expr $(; $b:expr)?),* $(,)?), $check:block, $expected_instruction_count:expr) => {
+    ($executable:expr, $mem:tt, ($($location:expr => $syscall_function:expr; $syscall_context_object:expr),* $(,)?), $check:block, $expected_instruction_count:expr) => {
         let check_closure = $check;
         let mut syscall_registry = SyscallRegistry::default();
-        $(test_interpreter_and_jit!(register, $executable, syscall_registry, $location => $a $(; $b)?);)*
+        $(test_interpreter_and_jit!(register, $executable, syscall_registry, $location => $syscall_function; $syscall_context_object);)*
         $executable.set_syscall_registry(syscall_registry);
         let (instruction_count_interpreter, _tracer_interpreter) = {
             let mut mem = $mem;
             let mut vm = EbpfVm::new($executable.as_ref(), &mut mem, &[]).unwrap();
-            $(test_interpreter_and_jit!(bind, vm, $location => $a $(; $b)?);)*
+            $(test_interpreter_and_jit!(bind, vm, $location => $syscall_function; $syscall_context_object);)*
             let result = vm.execute_program_interpreted(&mut TestInstructionMeter { remaining: $expected_instruction_count });
             assert!(check_closure(&vm, result));
             (vm.get_total_instruction_count(), vm.get_tracer().clone())
@@ -62,16 +58,14 @@ macro_rules! test_interpreter_and_jit {
             match compilation_result {
                 Err(err) => assert!(check_closure(&vm, Err(err))),
                 Ok(()) => {
-                    $(test_interpreter_and_jit!(bind, vm, $location => $a $(; $b)?);)*
+                    $(test_interpreter_and_jit!(bind, vm, $location => $syscall_function; $syscall_context_object);)*
                     let result = vm.execute_program_jit(&mut TestInstructionMeter { remaining: $expected_instruction_count });
                     let tracer_jit = vm.get_tracer();
                     if !check_closure(&vm, result) || !solana_rbpf::vm::Tracer::compare(&_tracer_interpreter, tracer_jit) {
-                        let mut tracer_display = String::new();
-                        _tracer_interpreter.write(&mut tracer_display, vm.get_program()).unwrap();
-                        println!("{}", tracer_display);
-                        let mut tracer_display = String::new();
-                        tracer_jit.write(&mut tracer_display, vm.get_program()).unwrap();
-                        println!("{}", tracer_display);
+                        let analysis = solana_rbpf::static_analysis::Analysis::from_executable($executable.as_ref());
+                        let stdout = std::io::stdout();
+                        _tracer_interpreter.write(&mut stdout.lock(), &analysis).unwrap();
+                        tracer_jit.write(&mut stdout.lock(), &analysis).unwrap();
                         panic!();
                     }
                     if $executable.get_config().enable_instruction_meter {
@@ -88,22 +82,21 @@ macro_rules! test_interpreter_and_jit {
 }
 
 macro_rules! test_interpreter_and_jit_asm {
-    ($source:tt, $mem:tt, ($($location:expr => $a:expr $(; $b:expr)?),* $(,)?), $check:block, $expected_instruction_count:expr) => {
-        let program = assemble($source).unwrap();
+    ($source:tt, $mem:tt, ($($location:expr => $syscall_function:expr; $syscall_context_object:expr),* $(,)?), $check:block, $expected_instruction_count:expr) => {
         #[allow(unused_mut)]
         {
             let config = Config {
                 enable_instruction_tracing: true,
                 ..Config::default()
             };
-            let mut executable = <dyn Executable::<UserError, TestInstructionMeter>>::from_text_bytes(&program, None, config).unwrap();
-            test_interpreter_and_jit!(executable, $mem, ($($location => $a $(; $b)?),*), $check, $expected_instruction_count);
+            let mut executable = assemble($source, None, config).unwrap();
+            test_interpreter_and_jit!(executable, $mem, ($($location => $syscall_function; $syscall_context_object),*), $check, $expected_instruction_count);
         }
     };
 }
 
 macro_rules! test_interpreter_and_jit_elf {
-    ($source:tt, $mem:tt, ($($location:expr => $a:expr $(; $b:expr)?),* $(,)?), $check:block, $expected_instruction_count:expr) => {
+    ($source:tt, $mem:tt, ($($location:expr => $syscall_function:expr; $syscall_context_object:expr),* $(,)?), $check:block, $expected_instruction_count:expr) => {
         let mut file = File::open($source).unwrap();
         let mut elf = Vec::new();
         file.read_to_end(&mut elf).unwrap();
@@ -114,7 +107,7 @@ macro_rules! test_interpreter_and_jit_elf {
                 ..Config::default()
             };
             let mut executable = <dyn Executable::<UserError, TestInstructionMeter>>::from_elf(&elf, None, config).unwrap();
-            test_interpreter_and_jit!(executable, $mem, ($($location => $a $(; $b)?),*), $check, $expected_instruction_count);
+            test_interpreter_and_jit!(executable, $mem, ($($location => $syscall_function; $syscall_context_object),*), $check, $expected_instruction_count);
         }
     };
 }
@@ -2199,19 +2192,19 @@ fn test_stack2() {
         mov r1, r10
         mov r2, 0x4
         sub r1, r2
-        call 1
+        syscall BpfMemFrob
         mov r1, 0
         ldxb r2, [r10-4]
         ldxb r3, [r10-3]
         ldxb r4, [r10-2]
         ldxb r5, [r10-1]
-        call 0
+        syscall BpfGatherBytes
         xor r0, 0x2a2a2a2a
         exit",
         [],
         (
-            0 => syscalls::BpfGatherBytes::call; syscalls::BpfGatherBytes {},
-            1 => syscalls::BpfMemFrob::call; syscalls::BpfMemFrob {},
+            b"BpfMemFrob" => syscalls::BpfMemFrob::call; syscalls::BpfMemFrob {},
+            b"BpfGatherBytes" => syscalls::BpfGatherBytes::call; syscalls::BpfGatherBytes {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0x01020304 } },
         16
@@ -2232,7 +2225,7 @@ fn test_string_stack() {
         mov r1, r10
         add r1, -8
         mov r2, r1
-        call 0
+        syscall BpfStrCmp
         mov r1, r0
         mov r0, 0x1
         lsh r1, 0x20
@@ -2242,7 +2235,7 @@ fn test_string_stack() {
         add r1, -8
         mov r2, r10
         add r2, -16
-        call 0
+        syscall BpfStrCmp
         mov r1, r0
         lsh r1, 0x20
         rsh r1, 0x20
@@ -2252,7 +2245,7 @@ fn test_string_stack() {
         exit",
         [],
         (
-            0 => syscalls::BpfStrCmp::call; syscalls::BpfStrCmp {},
+            b"BpfStrCmp" => syscalls::BpfStrCmp::call; syscalls::BpfStrCmp {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0x0 } },
         28
@@ -2287,7 +2280,7 @@ fn test_relative_call() {
         "tests/elfs/relative_call.so",
         [1],
         (
-            hash_symbol_name(b"log") => BpfSyscallString::call; BpfSyscallString {},
+            b"log" => BpfSyscallString::call; BpfSyscallString {},
         ),
         { |_vm, res: Result| { res.unwrap() == 2 } },
         14
@@ -2300,7 +2293,7 @@ fn test_bpf_to_bpf_scratch_registers() {
         "tests/elfs/scratch_registers.so",
         [1],
         (
-            hash_symbol_name(b"log_64") => BpfSyscallU64::call; BpfSyscallU64 {},
+            b"log_64" => BpfSyscallU64::call; BpfSyscallU64 {},
         ),
         { |_vm, res: Result| { res.unwrap() == 112 } },
         41
@@ -2325,12 +2318,12 @@ fn test_syscall_parameter_on_stack() {
         mov64 r1, r10
         add64 r1, -0x100
         mov64 r2, 0x1
-        call 0
+        syscall BpfSyscallString
         mov64 r0, 0x0
         exit",
         [],
         (
-            0 => BpfSyscallString::call; BpfSyscallString {},
+            b"BpfSyscallString" => BpfSyscallString::call; BpfSyscallString {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0 } },
         6
@@ -2402,22 +2395,35 @@ fn test_err_callx_oob_high() {
 
 #[test]
 fn test_err_call_lddw() {
-    test_interpreter_and_jit_asm!(
-        "
-        call 2
-        lddw r0, 0x1122334455667788
-        exit",
-        [],
-        (2 => 0x2),
-        {
-            |_vm, res: Result| {
-                matches!(res.unwrap_err(),
-                    EbpfError::UnsupportedInstruction(pc) if pc == 31
-                )
-            }
-        },
-        2
-    );
+    #[allow(unused_mut)]
+    {
+        let config = Config {
+            enable_instruction_tracing: true,
+            ..Config::default()
+        };
+        let mut executable = assemble(
+            "
+            call 1
+            lddw r0, 0x1122334455667788
+            exit",
+            None,
+            config,
+        )
+        .unwrap();
+        test_interpreter_and_jit!(
+            executable,
+            [],
+            (),
+            {
+                |_vm, res: Result| {
+                    matches!(res.unwrap_err(),
+                        EbpfError::UnsupportedInstruction(pc) if pc == 31
+                    )
+                }
+            },
+            2
+        );
+    }
 }
 
 #[test]
@@ -2451,7 +2457,7 @@ fn test_bpf_to_bpf_depth() {
             "tests/elfs/multiple_file.so",
             [i as u8],
             (
-                hash_symbol_name(b"log") => BpfSyscallString::call; BpfSyscallString {},
+                b"log" => BpfSyscallString::call; BpfSyscallString {},
             ),
             { |_vm, res: Result| { res.unwrap() == 0 } },
             if i == 0 { 4 } else { 3 + 10 * i as u64 }
@@ -2466,7 +2472,7 @@ fn test_err_bpf_to_bpf_too_deep() {
         "tests/elfs/multiple_file.so",
         [config.max_call_depth as u8],
         (
-            hash_symbol_name(b"log") => BpfSyscallString::call; BpfSyscallString {},
+            b"log" => BpfSyscallString::call; BpfSyscallString {},
         ),
         {
             |_vm, res: Result| {
@@ -2535,12 +2541,12 @@ fn test_err_syscall_string() {
     test_interpreter_and_jit_asm!(
         "
         mov64 r1, 0x0
-        call 0
+        syscall BpfSyscallString
         mov64 r0, 0x0
         exit",
         [72, 101, 108, 108, 111],
         (
-            0 => BpfSyscallString::call; BpfSyscallString {},
+            b"BpfSyscallString" => BpfSyscallString::call; BpfSyscallString {},
         ),
         {
             |_vm, res: Result| {
@@ -2559,12 +2565,12 @@ fn test_syscall_string() {
     test_interpreter_and_jit_asm!(
         "
         mov64 r2, 0x5
-        call 0
+        syscall BpfSyscallString
         mov64 r0, 0x0
         exit",
         [72, 101, 108, 108, 111],
         (
-            0 => BpfSyscallString::call; BpfSyscallString {},
+            b"BpfSyscallString" => BpfSyscallString::call; BpfSyscallString {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0 } },
         4
@@ -2580,12 +2586,12 @@ fn test_syscall() {
         mov64 r3, 0xCC
         mov64 r4, 0xDD
         mov64 r5, 0xEE
-        call 0
+        syscall BpfSyscallU64
         mov64 r0, 0x0
         exit",
         [],
         (
-            0 => BpfSyscallU64::call; BpfSyscallU64 {},
+            b"BpfSyscallU64" => BpfSyscallU64::call; BpfSyscallU64 {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0 } },
         8
@@ -2601,11 +2607,11 @@ fn test_call_gather_bytes() {
         mov r3, 3
         mov r4, 4
         mov r5, 5
-        call 0
+        syscall BpfGatherBytes
         exit",
         [],
         (
-            0 => syscalls::BpfGatherBytes::call; syscalls::BpfGatherBytes {},
+            b"BpfGatherBytes" => syscalls::BpfGatherBytes::call; syscalls::BpfGatherBytes {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0x0102030405 } },
         7
@@ -2619,7 +2625,7 @@ fn test_call_memfrob() {
         mov r6, r1
         add r1, 2
         mov r2, 4
-        call 0
+        syscall BpfMemFrob
         ldxdw r0, [r6]
         be64 r0
         exit",
@@ -2627,7 +2633,7 @@ fn test_call_memfrob() {
             0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, //
         ],
         (
-            0 => syscalls::BpfMemFrob::call; syscalls::BpfMemFrob {},
+            b"BpfMemFrob" => syscalls::BpfMemFrob::call; syscalls::BpfMemFrob {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0x102292e2f2c0708 } },
         7
@@ -2643,12 +2649,12 @@ fn test_syscall_with_context() {
         mov64 r3, 0xCC
         mov64 r4, 0xDD
         mov64 r5, 0xEE
-        call 0
+        syscall SyscallWithContext
         mov64 r0, 0x0
         exit",
         [],
         (
-            0 => SyscallWithContext::call; SyscallWithContext { context: 42 },
+            b"SyscallWithContext" => SyscallWithContext::call; SyscallWithContext { context: 42 },
         ),
         { |vm: &EbpfVm<UserError, TestInstructionMeter>, res: Result| {
             let syscall_context_object = unsafe { &*(vm.get_syscall_context_object(SyscallWithContext::call as usize).unwrap() as *const SyscallWithContext) };
@@ -2667,8 +2673,8 @@ fn test_load_elf() {
         "tests/elfs/noop.so",
         [],
         (
-            hash_symbol_name(b"log") => BpfSyscallString::call; BpfSyscallString {},
-            hash_symbol_name(b"log_64") => BpfSyscallU64::call; BpfSyscallU64 {},
+            b"log" => BpfSyscallString::call; BpfSyscallString {},
+            b"log_64" => BpfSyscallU64::call; BpfSyscallU64 {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0 } },
         11
@@ -2681,7 +2687,7 @@ fn test_load_elf_empty_noro() {
         "tests/elfs/noro.so",
         [],
         (
-            hash_symbol_name(b"log_64") => BpfSyscallU64::call; BpfSyscallU64 {},
+            b"log_64" => BpfSyscallU64::call; BpfSyscallU64 {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0 } },
         8
@@ -2694,7 +2700,7 @@ fn test_load_elf_empty_rodata() {
         "tests/elfs/empty_rodata.so",
         [],
         (
-            hash_symbol_name(b"log_64") => BpfSyscallU64::call; BpfSyscallU64 {},
+            b"log_64" => BpfSyscallU64::call; BpfSyscallU64 {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0 } },
         8
@@ -2714,8 +2720,8 @@ fn test_custom_entrypoint() {
         executable,
         [],
         (
-            hash_symbol_name(b"log") => BpfSyscallString::call; BpfSyscallString {},
-            hash_symbol_name(b"log_64") => BpfSyscallU64::call; BpfSyscallU64 {},
+            b"log" => BpfSyscallString::call; BpfSyscallString {},
+            b"log_64" => BpfSyscallU64::call; BpfSyscallU64 {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0 } },
         2
@@ -2768,11 +2774,12 @@ fn test_tight_infinite_loop_unconditional() {
 fn test_tight_infinite_recursion() {
     test_interpreter_and_jit_asm!(
         "
+        entrypoint:
         mov64 r3, 0x41414141
-        call 0
+        call entrypoint
         exit",
         [],
-        (0 => 0x0),
+        (),
         {
             |_vm, res: Result| {
                 matches!(res.unwrap_err(),
@@ -2814,12 +2821,12 @@ fn test_instruction_count_syscall() {
     test_interpreter_and_jit_asm!(
         "
         mov64 r2, 0x5
-        call 0
+        syscall BpfSyscallString
         mov64 r0, 0x0
         exit",
         [72, 101, 108, 108, 111],
         (
-            0 => BpfSyscallString::call; BpfSyscallString {},
+            b"BpfSyscallString" => BpfSyscallString::call; BpfSyscallString {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0 } },
         4
@@ -2836,7 +2843,7 @@ fn test_err_instruction_count_syscall_capped() {
         exit",
         [72, 101, 108, 108, 111],
         (
-            0 => BpfSyscallString::call; BpfSyscallString {},
+            b"BpfSyscallString" => BpfSyscallString::call; BpfSyscallString {},
         ),
         {
             |_vm, res: Result| {
@@ -2860,7 +2867,7 @@ fn test_non_terminate_early() {
         mov64 r3, 0x0
         mov64 r4, 0x0
         mov64 r5, r6
-        call 0x6
+        syscall Unresolved
         add64 r6, 0x1
         ja -0x8
         exit",
@@ -2888,13 +2895,13 @@ fn test_err_non_terminate_capped() {
         mov64 r3, 0x0
         mov64 r4, 0x0
         mov64 r5, r6
-        call 0x0
+        syscall BpfTracePrintf
         add64 r6, 0x1
         ja -0x8
         exit",
         [],
         (
-            0 => BpfTracePrintf::call; BpfTracePrintf {},
+            b"BpfTracePrintf" => BpfTracePrintf::call; BpfTracePrintf {},
         ),
         {
             |_vm, res: Result| {
@@ -2918,13 +2925,13 @@ fn test_err_non_terminating_capped() {
         mov64 r3, 0x0
         mov64 r4, 0x0
         mov64 r5, r6
-        call 0x0
+        syscall BpfTracePrintf
         add64 r6, 0x1
         ja -0x8
         exit",
         [],
         (
-            0 => BpfTracePrintf::call; BpfTracePrintf {},
+            b"BpfTracePrintf" => BpfTracePrintf::call; BpfTracePrintf {},
         ),
         {
             |_vm, res: Result| {
@@ -2947,12 +2954,12 @@ fn test_symbol_relocation() {
         mov64 r1, r10
         sub64 r1, 0x1
         mov64 r2, 0x1
-        call 0
+        syscall BpfSyscallString
         mov64 r0, 0x0
         exit",
         [72, 101, 108, 108, 111],
         (
-            0 => BpfSyscallString::call; BpfSyscallString {},
+            b"BpfSyscallString" => BpfSyscallString::call; BpfSyscallString {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0 } },
         6
@@ -2963,7 +2970,7 @@ fn test_symbol_relocation() {
 fn test_err_symbol_unresolved() {
     test_interpreter_and_jit_asm!(
         "
-        call 0
+        syscall Unresolved
         mov64 r0, 0x0
         exit",
         [],
@@ -2984,7 +2991,7 @@ fn test_err_call_unresolved() {
         mov r3, 3
         mov r4, 4
         mov r5, 5
-        call 63
+        syscall Unresolved
         exit",
         [],
         (),
@@ -3001,7 +3008,7 @@ fn test_err_unresolved_elf() {
         "tests/elfs/unresolved_syscall.so",
         [],
         (
-            hash_symbol_name(b"log") => BpfSyscallString::call; BpfSyscallString {},
+            b"log" => BpfSyscallString::call; BpfSyscallString {},
         ),
         {
             |_vm, res: Result| matches!(res.unwrap_err(), EbpfError::ElfError(ElfError::UnresolvedSymbol(symbol, pc, offset)) if symbol == "log_64" && pc == 550 && offset == 4168)
@@ -3220,32 +3227,36 @@ fn test_tcp_sack_nomatch() {
 
 #[test]
 fn test_large_program() {
-    fn write_insn(prog: &mut [u8], index: usize, asm: &str) {
-        let insn = assemble(asm).unwrap();
-        prog[index * ebpf::INSN_SIZE..index * ebpf::INSN_SIZE + insn.len()].copy_from_slice(&insn);
+    let mut insns = vec![ebpf::Insn::default(); ebpf::PROG_MAX_INSNS * 2 - 1];
+    for index in (0..(ebpf::PROG_MAX_INSNS * 2 - 1)).step_by(2) {
+        insns[index].opc = ebpf::LD_DW_IMM;
     }
-
-    let mut prog = vec![0; (ebpf::PROG_MAX_INSNS * 2 - 1) * ebpf::INSN_SIZE];
-    let mut insn = vec![0; ebpf::INSN_SIZE * 2];
-    write_insn(&mut insn, 0, "lddw r0, 0");
-    for index in (0..(ebpf::PROG_MAX_INSNS - 1) * ebpf::INSN_SIZE * 2).step_by(ebpf::INSN_SIZE * 2)
-    {
-        prog[index..index + ebpf::INSN_SIZE * 2].copy_from_slice(&insn);
-    }
-    write_insn(&mut prog, ebpf::PROG_MAX_INSNS - 4, "ja 9");
-    write_insn(&mut prog, ebpf::PROG_MAX_INSNS - 3, "mov r0, 0");
-    write_insn(&mut prog, ebpf::PROG_MAX_INSNS * 2 - 2, "exit");
+    insns[ebpf::PROG_MAX_INSNS - 4].opc = ebpf::JA;
+    insns[ebpf::PROG_MAX_INSNS - 4].off = 9;
+    insns[ebpf::PROG_MAX_INSNS - 3].opc = ebpf::MOV64_IMM;
+    insns[ebpf::PROG_MAX_INSNS * 2 - 2].opc = ebpf::EXIT;
 
     let config = Config {
         enable_instruction_tracing: true,
         ..Config::default()
     };
 
+    let mut program = insns
+        .iter()
+        .map(|insn| insn.to_vec())
+        .flatten()
+        .collect::<Vec<_>>();
     #[allow(unused_mut)]
     {
-        let mut executable =
-            <dyn Executable<UserError, TestInstructionMeter>>::from_text_bytes(&prog, None, config)
-                .unwrap();
+        let mut bpf_functions = BTreeMap::new();
+        register_bpf_function(&mut bpf_functions, 0, "entrypoint").unwrap();
+        let mut executable = <dyn Executable<UserError, TestInstructionMeter>>::from_text_bytes(
+            program.as_slice(),
+            bpf_functions,
+            None,
+            config,
+        )
+        .unwrap();
         test_interpreter_and_jit!(
             executable,
             [],
@@ -3257,11 +3268,15 @@ fn test_large_program() {
 
     {
         // Test program that is too large
-        prog.extend_from_slice(&assemble("exit").unwrap());
+        let exit_insn = program[(ebpf::PROG_MAX_INSNS * 2 - 2) * ebpf::INSN_SIZE
+            ..(ebpf::PROG_MAX_INSNS * 2 - 1) * ebpf::INSN_SIZE]
+            .to_vec();
+        program.extend_from_slice(exit_insn.as_slice());
 
         assert!(
             <dyn Executable::<UserError, DefaultInstructionMeter>>::from_text_bytes(
-                &prog,
+                program.as_slice(),
+                BTreeMap::new(),
                 Some(check),
                 config,
             )
@@ -3276,18 +3291,22 @@ fn test_large_program() {
 fn execute_generated_program(prog: &[u8]) -> bool {
     let max_instruction_count = 1024;
     let mem_size = 1024 * 1024;
+    let mut bpf_functions = BTreeMap::new();
+    register_bpf_function(&mut bpf_functions, 0, "entrypoint").unwrap();
     let executable = <dyn Executable<UserError, TestInstructionMeter>>::from_text_bytes(
         &prog,
+        bpf_functions,
         Some(check),
         Config {
             enable_instruction_tracing: true,
             ..Config::default()
         },
     );
-    if executable.is_err() {
+    let mut executable = if let Ok(executable) = executable {
+        executable
+    } else {
         return false;
-    }
-    let mut executable = executable.unwrap();
+    };
     executable.set_syscall_registry(SyscallRegistry::default());
     if executable.jit_compile().is_err() {
         return false;
@@ -3314,18 +3333,14 @@ fn execute_generated_program(prog: &[u8]) -> bool {
     if result_interpreter != result_jit
         || !solana_rbpf::vm::Tracer::compare(&tracer_interpreter, tracer_jit)
     {
+        let analysis = solana_rbpf::static_analysis::Analysis::from_executable(executable.as_ref());
         println!("result_interpreter={:?}", result_interpreter);
         println!("result_jit={:?}", result_jit);
-        let mut tracer_display = String::new();
+        let stdout = std::io::stdout();
         tracer_interpreter
-            .write(&mut tracer_display, vm.get_program())
+            .write(&mut stdout.lock(), &analysis)
             .unwrap();
-        println!("{}", tracer_display);
-        let mut tracer_display = String::new();
-        tracer_jit
-            .write(&mut tracer_display, vm.get_program())
-            .unwrap();
-        println!("{}", tracer_display);
+        tracer_jit.write(&mut stdout.lock(), &analysis).unwrap();
         panic!();
     }
     if executable.get_config().enable_instruction_meter {
